import java.awt.Container;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JOptionPane;
import javax.swing.JPanel;
import javax.swing.JTextArea;

public class ArrayFrame extends JFrame implements ActionListener {
    private JButton revertierenButton;
    private JButton lexVergleichButton;

    private JTextArea ausgabeBereich;

    /**
     * In diesem Programmstück wird das Fenster erzeugt
     */
    public ArrayFrame() {
        /* In der Kopfleiste des Fenster steht "ArrayFrame" */
        this.setTitle("ArrayFrame");

        /* Das Fenster ist 500 Pixel breit und 200 Pixel hoch. */
        this.setSize(500, 200);

        /* Hier werden alle Buttons erzeugt. */
        this.revertierenButton = new JButton("Array revertieren");
        this.lexVergleichButton = new JButton(
                "Zwei Arrays lexikographisch vergleichen");

        /* Hier wird der Ausgabe-Bereich erzeugt. */
        this.ausgabeBereich = new JTextArea(10, 100);

        /* Hier werden alle Buttons zusammengruppiert. */
        JPanel buttonPanel = new JPanel();
        buttonPanel.setLayout(new GridLayout(2, 1));
        buttonPanel.add(this.revertierenButton);
        buttonPanel.add(this.lexVergleichButton);

        /*
         * Der ContentPane ist der Ausschnitt des Fensters, auf dem Widgets d.h.
         * Interaktionselemente (wie eine TextArea oder ein Button) platziert
         * werden können.
         */
        Container contentPane = this.getContentPane();
        contentPane.setLayout(new GridLayout(2, 1));
        /* Hier wird die Gruppe von Buttons platziert. */
        contentPane.add(buttonPanel);
        /* Hier wird der Ausgabebereich platziert. */
        contentPane.add(this.ausgabeBereich);

        /*
         * Hier wird der ArrayFrame als Listener für Knopfdruck-Ereignisse bei
         * jedem der Buttons registriert.
         */
        this.revertierenButton.addActionListener(this);
        this.lexVergleichButton.addActionListener(this);

        /*
         * Wird das Fenster geschlossen (d.h. auf X gedrückt), wird mit Hilfe
         * dieser Programmzeile auch unser Programm ordnungsgemäß beendet.
         */
        this.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // je nach Button entsprechende Methode ausführen
        Object source = e.getSource();
        if (source == this.revertierenButton) {
            this.revertierung();
        }
        else if (source == this.lexVergleichButton) {
            this.lexikografischeOrdnung();
        }
    }

    /**
     * implementiert die Funktionalität des Revertierungsbuttons
     */
    private void revertierung() {
        String einlesenArray = JOptionPane.showInputDialog("Array: ");
        int[] array = Konverter.konvertiereZuIntArray(einlesenArray);

        for (int i = 0; i < array.length / 2; i++) {
            int temp = array[i];
            array[i] = array[(array.length - 1) - i];
            array[(array.length - 1) - i] = temp;
        }

        String eingabe = "Eingabe: " + einlesenArray + "\n\n";
        String ergebnis = eingabe + "Ausgabe: "
                + Konverter.konvertiereZuString(array);
        this.ausgabeBereich.setText(ergebnis);
    }

    /**
     * implementiert die Funktionalität des Lexikografische-Ordnung-Button
     */
    private void lexikografischeOrdnung() {
        String einlesenArrayA = JOptionPane
                .showInputDialog("Erstes Array: ");
        int[] arrayA = Konverter
                .konvertiereZuIntArray(einlesenArrayA);

        String einlesenArrayB = JOptionPane
                .showInputDialog("Zweites Array: ");
        int[] arrayB = Konverter
                .konvertiereZuIntArray(einlesenArrayB);

        String eingabe = "Erstes Array: " + einlesenArrayA + "\n";
        eingabe = eingabe + "Zweites Array: " + einlesenArrayB
                + "\n\n";

        boolean istLexKleiner = istLexikographischKleiner(arrayA,
                arrayB);

        String ergebnis = eingabe;
        if (istLexKleiner) {
            ergebnis = eingabe
                    + "Das erste Array ist lexikografisch kleiner als das zweite.";

        }
        else {
            ergebnis = eingabe
                    + "Das zweite Array ist lexikografisch kleiner als das erste.";
        }
        this.ausgabeBereich.setText(ergebnis);
    }

    /**
     * testet, ob das erste Array lexikografisch kleiner als das zweite ist.
     * 
     * @param arrayA
     * @param arrayB
     * @return
     */
    private boolean istLexikographischKleiner(int[] arrayA,
            int[] arrayB) {
        int minLength = 0;
        if (arrayA.length < arrayB.length) {
            minLength = arrayA.length;
        }
        else {
            minLength = arrayB.length;
        }
        // alternativ: minLength = (int)Math.min(arr1.length, arr2.length);
        for (int i = 0; i < minLength; i++) {
            if (arrayA[i] < arrayB[i]) { // arr1 ist kleiner als arr2
                return true;
            }
            else if (arrayA[i] > arrayB[i]) { // arr1 ist groesser als arr2
                return false;
            }
            // die Elemente sind gleich, pruefe naechstes
        }
        // beide Arrays sind, bei den vergleichbaren
        // Elementen, gleich
        return arrayA.length < arrayB.length; // dann ist arr1 nur kleiner, wenn kuerzer
    }
}
